from typing import Dict


def merge_dicts(dict1: Dict, dict2: Dict) -> Dict:
    context = dict(dict1)
    for k, v2 in dict2.items():
        v1 = context.get(k)
        if not context.get(k):
            context[k] = v2
        elif isinstance(v1, list) and isinstance(v2, list):
            context[k] = v1 + v2
        elif isinstance(v1, dict) and isinstance(v2, dict):
            context[k] = merge_dicts(v1, v2)
        else:
            context[k] = v2

    return context



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from typing import Optional as typing___Optional
from typing import Text as typing___Text

from google.protobuf.descriptor import Descriptor as google___protobuf___descriptor___Descriptor
from google.protobuf.descriptor import FileDescriptor as google___protobuf___descriptor___FileDescriptor
from google.protobuf.message import Message as google___protobuf___message___Message
from typing_extensions import Literal as typing_extensions___Literal

builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int


DESCRIPTOR: google___protobuf___descriptor___FileDescriptor = ...

class Service(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...
    uuid: typing___Text = ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        uuid : typing___Optional[typing___Text] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"name",b"name",u"uuid",b"uuid"]) -> None: ...
type___Service = Service

class Metadata(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    message_uuid: typing___Text = ...
    protocol_version: typing___Text = ...
    timestamp: builtin___float = ...
    topic: typing___Text = ...
    data_encoding: typing___Text = ...

    def __init__(self,
        *,
        message_uuid : typing___Optional[typing___Text] = None,
        protocol_version : typing___Optional[typing___Text] = None,
        timestamp : typing___Optional[builtin___float] = None,
        topic : typing___Optional[typing___Text] = None,
        data_encoding : typing___Optional[typing___Text] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"data_encoding",b"data_encoding",u"message_uuid",b"message_uuid",u"protocol_version",b"protocol_version",u"timestamp",b"timestamp",u"topic",b"topic"]) -> None: ...
type___Metadata = Metadata

class SNSSQSMessage(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    data: builtin___bytes = ...

    @property
    def service(self) -> type___Service: ...

    @property
    def metadata(self) -> type___Metadata: ...

    def __init__(self,
        *,
        service : typing___Optional[type___Service] = None,
        metadata : typing___Optional[type___Metadata] = None,
        data : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"metadata",b"metadata",u"service",b"service"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"data",b"data",u"metadata",b"metadata",u"service",b"service"]) -> None: ...
type___SNSSQSMessage = SNSSQSMessage



import logging
from typing import Any, Dict  # noqa

from tomodachi.transport.aws_sns_sqs import aws_sns_sqs_publish


# An example discovery class which would send a message over AWS SNS on the topic
# 'services-registration-register' containing the newly started service-name and
# the HTTP endpoitns that it's listening on. Likewise it also tries to "deregister"
# by sending a message on the 'services-registration-deregister' when shutting
# down.
class AWSSNSRegistration(object):
    http_endpoints: Dict = {}

    @classmethod
    async def add_http_endpoint(cls, service: Any, host: str, port: int, method: str, pattern: str) -> None:
        cls.http_endpoints[service] = cls.http_endpoints.get(service, [])
        cls.http_endpoints[service].append((host, port, method, pattern))

    @classmethod
    async def _register_service(cls, service: Any) -> None:
        logging.getLogger("discovery.aws_sns_registration").info(
            'Registering service "{}" [id: {}]'.format(service.name, service.uuid)
        )
        data = {"name": service.name, "uuid": service.uuid, "http_endpoints": cls.http_endpoints.get(service)}
        await aws_sns_sqs_publish(service, data, topic="services-registration-register")

    @classmethod
    async def _deregister_service(cls, service: Any) -> None:
        logging.getLogger("discovery.aws_sns_registration").info(
            'Deregistering service "{}" [id: {}]'.format(service.name, service.uuid)
        )
        data = {"name": service.name, "uuid": service.uuid}
        try:
            await aws_sns_sqs_publish(service, data, topic="services-registration-deregister")
        except Exception:
            logging.getLogger("discovery.aws_sns_registration").info(
                'Deregistering service "{}" failed [id: {}]'.format(service.name, service.uuid)
            )
